# define COMMENT 257
# define STRING 258
# define PREAMBLE 259
# define ARTICLE 260
# define BOOK 261
# define BOOKLET 262
# define CONFERENCE 263
# define INBOOK 264
# define INCOLLECTION 265
# define INPROCEEDINGS 266
# define MANUAL 267
# define MASTERSTHESIS 268
# define MISC 269
# define PHDTHESIS 270
# define PROCEEDINGS 271
# define TECHREPORT 272
# define UNPUBLISHED 273
# define TYPE 274
# define KOMMA 275
# define EQ 276
# define LGKL 277
# define RGKL 278
# define LRKL 279
# define RRKL 280
# define DAZ 281
# define ZAUN 282
# define PROZENT 283
# define ZEICHEN 284
# define WHITE_SPACE 285
# define BUCHSTABE_ZAHL 286
# define KOMMENTAR 287
