"BibView.title:		bibView",
"BibView.iconName:		bibView",
"BibView*font: *helvetica-bold-r-*-120-*",
"BibView*mainwin.orientation:		vertical",
"BibView*mainwin.height:			150",
"BibView*mainwin.width:			260",
"BibView*mainwin*showGrip:		false",
"BibView*mainwin.commandBox.width: 	500",
"BibView*mainwin.commandBox.vSpace: 	0",
"BibView*mainwin.commandBox.hSpace: 	0",
"BibView*mainwin.commandBox.min: 	21",
"BibView*mainwin.desktop.label:",
"BibView*mainwin.desktop.width:   	500",
"BibView*mainwin.desktop.height:    	150",
"BibView*mainwin.desktop.fromVert:  	commandBox",
"BibView*mainwin*file.label:		Datei",
"BibView*mainwin*file.menuName:		fileMenu",
"BibView*fileMenu.label:			Datei-Menue",
"BibView*fileMenu*item1*label:		Neu",
"BibView*fileMenu*item2*label:		Oeffnen",
"BibView*fileMenu*item3*label:		Schliessen",
"BibView*fileMenu*item4*label:		Speichern",
"BibView*fileMenu*item5*label:		Speichern unter",
"BibView*fileMenu*quit*label:		Beenden",
"BibView*mainwin*services.label:      	Dienste",
"BibView*mainwin*services.menuName:	servicesMenu",
"BibView*servicesMenu.label:		Dienste-Menue",
"BibView*servicesMenu*item1*label:	Konsistenzcheck	",
"BibView*servicesMenu*item2*label:	Mischen",
"BibView*servicesMenu*item3*label:	Druck in Datei",
"BibView*servicesMenu*item4*label:	Makros editieren",
"BibView*servicesMenu*item5*label:	Konfiguration laden",
"BibView*mainwin*options.label:       	Optionen	",
"BibView*mainwin*options.menuName:	optionsMenu",
"BibView*mainwin*options*leftMargin: 	20",
"BibView*optionsMenu.label:		Optionen-Menue",
"BibView*optionsMenu*item1*label:	Warnton bei Fehler",
"BibView*optionsMenu*item2*label:	Sicherungskopien  ",
"BibView*optionsMenu*item3*label:	Icons auf Desktop  ",
"BibView*optionsMenu*item4*label:	Autom. Ueberpruefung",
"BibView*optionsMenu*item5*label:	Zwingende Felder",
"BibView*optionsMenu*item6*label:	Ignoriere Gross/Klein",
"BibView*optionsMenu*item7*label:	Drucke als BibTeX",
"BibView*optionsMenu*item8*label:	Fehlerfenster",
"BibView*mainwin*window.label:      	Fenster",
"BibView*mainwin*window.menuName:	windowMenu",
"BibView*mainwin*window*leftMargin: 	20",
"BibView*windowMenu.label:		Fenster-Menue",
"BibView*windowMenu*item1*label:		Kaskadieren",
"BibView*mainwin*help.label:      	Hilfe",
"BibView*mainwin*help.menuName:		helpMenu",
"BibView*helpMenu.label:			Hilfe-Menue",
"BibView*helpMenu*item1*label:		Ueberblick",
"BibView*helpMenu*item2*label:		Hauptfenster",
"BibView*helpMenu*item3*label:		Bibliographiefenster",
"BibView*helpMenu*item4*label:		Listenfenster",
"BibView*helpMenu*item5*label:		Kartenfenster",
"BibView*helpMenu*item6*label:		Andere Fenster",
"BibView*helpMenu*info*label:		Programm-Info",
"BibView*bibWin.orientation:		vertical",
"BibView*bibWin.width:			440",
"BibView*bibWin.height:			150",
"BibView*bibWin*showGrip:		false",
"BibView*bibWin*commandBox.width: 	450",
"BibView*bibWin*commandBox.vSpace: 	0",
"BibView*bibWin*commandBox.hSpace: 	0",
"BibView*bibWin*commandBox.min: 		21",
"BibView*bibWin*commandBox*new.label:	Neu",
"BibView*bibWin*new.menuName:		newMenu",
"BibView*bibWin*newMenu.label:		Neu-Menue",
"BibView*bibWin*commandBox*search.label:	Suchen",
"BibView*bibWin*commandBox*list.label:	Liste",
"BibView*bibWin*commandBox*macro.label:	Makros",
"BibView*bibWin*commandBox*print.label:	Drucken",
"BibView*bibWin*commandBox*save.label:	Speichern",
"BibView*bibWin*commandBox*close.label:	Schliessen",
"BibView*bibWin*window.label:		Fenster",
"BibView*bibWin*window.menuName:		windowMenu",
"BibView*bibWin*window*leftMargin: 	20",
"BibView*bibWin*windowMenu.label:	Fenster-Menue",
"BibView*bibWin*windowMenu*item1.label:	Kaskadieren",
"BibView*bibWin.bibdesk.label:",
"BibView*bibWin.bibdesk.width:   	450",
"BibView*bibWin.bibdesk.height:    	350",
"BibView*bibWin.bibdesk.fromVert:  	commandBox",
"BibView*listWin*allowShellResize:	true",
"BibView*listWin*allowResize:		true",
"BibView*listWin.height:			650",
"BibView*listWin.width:			530",
"BibView*listWin.horizDistance:		235",
"BibView*listWin.vertDistance:		20",
"BibView*listWin.borderWidth:		2",
"BibView*listWin.orientation:		vertical",
"BibView*listWin*showGrip:		false",
"BibView*listWin.commandBox.vSpace: 	0",
"BibView*listWin.commandBox.hSpace: 	0",
"BibView*listWin.commandBox.min: 	21",
"BibView*listWin.commandBox*print.label:	Drucken",
"BibView*listWin.commandBox*save.label:	Speichern",
"BibView*listWin.commandBox*sort.label:	Sortieren",
"BibView*listWin.commandBox*copy.label:	Kopieren",
"BibView*listWin.commandBox*quit.label:	Schliessen",
"BibView*listWin*sort.menuName:		sortMenu",
"BibView*listWin*sortMenu.label:	Sortieren Menu",
"BibView*listWin*listVport.fromVert:	commandBox",
"BibView*listWin*listVport*font: 	*fixed-bold-r-*-100-*",
"BibView*listWin*head1.width:		1000",
"BibView*listWin*list.verticalList:	true",
"BibView*listWin*list.forceColumns:	true",
"BibView*listWin*list.defaultColumns:	1",
"BibView*macWin*allowResize:		true",
"BibView*macWin.height:			300",
"BibView*macWin.width:			450",
"BibView*macWin.borderWidth:		2",
"BibView*macWin.orientation:		vertical",
"BibView*macWin*showGrip:		false",
"BibView*macWin.commandBox.vSpace: 	0",
"BibView*macWin.commandBox.hSpace: 	0",
"BibView*macWin.commandBox.min: 	21",
"BibView*macWin.commandBox*save.label:	Speichern",
"BibView*macWin.commandBox*close.label:	Schliessen",
"BibView*macText.fromVert:		commandBox",
"BibView*macText*font: 			*fixed-bold-r-*-100-*",
"BibView*macText*textSource.editType:	edit",
"BibView*macText*input: 			true",
"BibView*helpWin*allowResize:		true",
"BibView*helpWin.height:			300",
"BibView*helpWin.width:			500",
"BibView*helpWin.borderWidth:		2",
"BibView*helpWin.orientation:		vertical",
"BibView*helpWin*showGrip:		false",
"BibView*helpWin.commandBox.vSpace: 	0",
"BibView*helpWin.commandBox.hSpace: 	0",
"BibView*helpWin.commandBox.min: 	21",
"BibView*helpWin.commandBox*close.label:	Schliessen",
"BibView*helpText.fromVert:		commandBox",
"BibView*helpText*font: 			*fixed-bold-r-*-100-*",
"BibView*helpText*input: 			true",
"BibView*annoteWin*allowResize:		true",
"BibView*annoteWin.height:			300",
"BibView*annoteWin.width:			500",
"BibView*annoteWin.borderWidth:		2",
"BibView*annoteWin.orientation:		vertical",
"BibView*annoteWin*showGrip:		false",
"BibView*annoteWin.commandBox.vSpace: 	0",
"BibView*annoteWin.commandBox.hSpace: 	0",
"BibView*annoteWin.commandBox.min: 	21",
"BibView*annoteWin.commandBox*close.label:	Schliessen",
"BibView*annoteWin.commandBox*save.label:	Speichern",
"BibView*annoteText.fromVert:		commandBox",
"BibView*annoteText*font: 			*fixed-bold-r-*-100-*",
"BibView*annoteText*input: 			true",
"BibView*srchShell.title:		Suchen Auswahl",
"BibView*srchWin.height:			400",
"BibView*srchWin.width:			380",
"BibView*srchWin.borderWidth:		2",
"BibView*srchWin.orientation:		vertical",
"BibView*srchWin*showGrip:		false",
"BibView*srchWin.commandBox.vSpace: 	0",
"BibView*srchWin.commandBox.hSpace: 	0",
"BibView*srchWin.commandBox.min: 	21",
"BibView*srchWin.commandBox*ok.label:	Suche starten",
"BibView*srchWin.commandBox*cancel.label:Abbrechen",
"BibView*srchWin*srchFlds*borderwidth:	0",
"BibView*srchWin*srchFlds*vSpace:	0",
"BibView*srchWin*srchFlds*font: 		*helvetica-bold-r-*-100-*",
"BibView*srchWin*ComboBox.borderWidth:	0",
"BibView*srchWin*ComboBox.margin:	4",
"BibView*srchWin*ComboBox*Text.width:	250",
"BibView*srchWin*ComboBox*Toggle.width:	100",
"BibView*srchWin*ComboBox*Toggle.justify:left",
"BibView*srchWin*ComboBox*Viewport.borderWidth:	4",
"BibView*srchWin*ComboBox*Viewport.width:	300",
"BibView*srchWin*ComboBox*Viewport.height:	100",
"BibView*srchWin*cbAllFields*name:	Beliebiges Feld:",
"BibView*srchWin*cbBibType*name:	BibTeX-Typ:",
"BibView*srchWin*cbMainkey*name:		BibTeX-Schluessel:",
"BibView*srchWin*cbUserfield1.width:    100",
"BibView*srchWin*cbUserfield2.width:    100",
"BibView*srchWin*cbUsercont1.width:     250",
"BibView*srchWin*cbUsercont2.width:     250",
"BibView*srchWin*fldHead.label:         Benutzerdefinierte Felder",
"BibView*srchWin*fldHead.width:         355",
"BibView*cardShell.allowShellResize:	true",
"BibView*cardWin*Text*scrollHorizontal:	whenNeeded",
"BibView*cardWin*Text*scrollVertical:	whenNeeded",
"BibView*cardWin*Text*thickness:		8",
"BibView*cardWin*Text.height:		28",
"BibView*cardWin*Text*textSource.editType:	edit",
"BibView*cardWin*Text*input: 		true",
"BibView*cardWin*Label*justify:		left",
"BibView*cardWin.width:			450",
"BibView*cardWin.borderWidth:		2",
"BibView*cardWin.commandBox.showGrip:	false",
"BibView*cardWin.commandBox.width: 	450",
"BibView*cardWin.commandBox.vSpace: 	0",
"BibView*cardWin.commandBox.hSpace: 	0",
"BibView*cardWin.commandBox.min: 	21",
"BibView*cardWin.commandBox.skipAdjust: 	true",
"BibView*cardWin.commandBox*userDef.label:	Benutzerdaten",
"BibView*cardWin.commandBox*annote.label:	Anmerkung",
"BibView*cardWin.commandBox*save.label:		Speichern",
"BibView*cardWin.commandBox*change.label:	Aendern",
"BibView*cardWin.commandBox*delete.label:	Loeschen",
"BibView*cardWin.commandBox*copy.label:		Kopieren",
"BibView*cardWin.commandBox*duplicate.label:	Verdoppeln",
"BibView*cardWin*duplicate.menuName:		duplicateMenu",
"BibView*cardWin*duplicateMenu.label:		Verdopplen Menu",
"BibView*cardWin.commandBox*quit.label:		Schliessen",
"BibView*cardWin.userFldVport*font: 	*helvetica-bold-r-*-100-*",
"BibView*cardWin.userFldVport.allowResize: true",
"BibView*cardWin.userFldVport.min:	100",
"BibView*cardWin.userFldVport.max:	150",
"BibView*cardWin*userFieldL.width:	150",
"BibView*cardWin*userField.width:	225",
"BibView*cardWin*userFldHead.label:	Benutzerdefinierte Felder",
"BibView*cardWin*userFldHead.width:	380",
"BibView*cardWin*userFldHead.borderWidth:	2",
"BibView*cardWin*userFldHead.justify:	center",
"BibView*cardWin.carddesk*font: 		*helvetica-bold-r-*-100-*",
"BibView*cardWin.carddesk*Label*borderWidth:	0",
"BibView*cardWin.carddesk.defaultDistance:	0",
"BibView*cardWin.carddesk*field.width:	        355",
"BibView*cardWin.carddesk*field.height:	        30",
/*
"BibView*cardWin*article.borderWidth:		0",
"BibView*cardWin*book.borderWidth:		0",
"BibView*cardWin*booklet.borderWidth:		0",
"BibView*cardWin*conference.borderWidth:		0",
"BibView*cardWin*inbook.borderWidth:		0",
"BibView*cardWin*incollection.borderWidth:	0",
"BibView*cardWin*inproceedings.borderWidth:	0",
"BibView*cardWin*manual.borderWidth:		0",
"BibView*cardWin*mastersthesis.borderWidth:	0",
"BibView*cardWin*misc.borderWidth:		0",
"BibView*cardWin*phdthesis.borderWidth:		0",
"BibView*cardWin*proceedings.borderWidth:	0",
"BibView*cardWin*techreport.borderWidth:		0",
"BibView*cardWin*unpublished.borderWidth:	0",
*/
"BibView*confirmLoadShell.title:		Laden?",
"BibView*confirmLoadBox.label:		Datei bereits geladen.\\nAuf gespeicherte Version zurueckgreifen?",
"BibView*confirmLoadBox*ok.label:	OK",
"BibView*confirmLoadBox*cancel.label:	Abbrechen",
"BibView*confirmQuitShell.title:	Schliessen?",
"BibView*confirmQuitBox.label:	Es wurden Aenderungen vorgenommen, die nicht gespeichert wurden!\\nWollen Sie wirklich alle Aenderungen verwerfen?",
"BibView*confirmQuitBox*ok.label:	Verwerfen",
"BibView*confirmQuitBox*cancel.label:	Abbrechen",
"BibView*fileSelectBoxShell.title:	BibView Dateimanager",
"BibView*fileSelectBoxShell.iconName:	BibView Dateimanager",
"BibView*fileLoadHead.label:		BibTeX-Datei auswaehlen:",
"BibView*fileRenameHead.label:		BibTeX-Datei umbenennen:",
"BibView*filePrintHead.label:		LaTeX Druckdatei:",
"BibView*configLoadHead.label:		Konfigurationsdatei:",
"BibView*fileListSaveHead.label:		Liste als BibTeX-Datei speichern:",
"BibView*bibselShell.title:		BibView-Datei-Auswahl",
"BibView*bibselWin*closeHead.label:	Schliessen von",
"BibView*bibselWin*saveHead.label:	Speichern von",
"BibView*bibselWin*saveAsHead.label:	Speichern unter ",
"BibView*bibselWin*copyHead.label:	Kopieren nach",
"BibView*bibselWin*collSrcHead.label:	Mischen aus",
"BibView*bibselWin*collDestHead.label:	Mischen nach",
"BibView*bibselWin*checkHead.label:	Ueberpruefen von",
"BibView*bibselWin*printHead.label:	Drucken von",
"BibView*bibselWin*macroHead.label:	Makros editieren von",
"BibView*bibselWin*cancel.label:		Abbrechen",
"BibView*errorShell.title:	BibView Nachricht",
"BibView*errorWin*type.label:	FEHLER:",
"BibView*errorWin*ok.label:	OK",
"BibView*warnShell.title:	BibView Nachricht",
"BibView*warnWin*type.label:	WARNUNG:",
"BibView*warnWin*ok.label:	OK",
"BibView*noticeShell.title:	BibView Nachricht",
"BibView*noticeWin*type.label:	HINWEIS:",
"BibView*noticeWin*ok.label:	OK",
"BibView*msgText10.label:  Dieser Befehl wurde noch nicht implementiert.",
"BibView*msgText11.label:  Aktion nicht ausgefuehrt. Kein Speicher!",
"BibView*msgText12.label:  Ausgewaehlte Datei nicht im BibTeX-Format!",
"BibView*msgText14.label:  Keine BibTeX-Datei geoeffnet zum Bearbeiten.",
"BibView*msgText15.label:  Datei kann nicht in sich selbst gemischt werden.",
"BibView*msgText16.label:  Karte kann nicht auf sich selbst kopiert werden.",
"BibView*msgText17.label:  Liste kann nicht auf sich selbst kopiert werden.",
"BibView*msgText30.label:  Fenster koennen nicht kaskadiert werden!",
"BibView*msgText50.label:  Datei kann nicht geoeffnet werden!",
"BibView*msgText51.label:  Keine Konfigurationsdatei angegeben.",
"BibView*msgText52.label:  Temporaerdatei kann nicht geoeffnet werden!",
"BibView*msgText53.label:  Macros koennen nicht gespeichert werden!",
"BibView*msgText54.label:  Bitte erst Error-Window schliessen!",
"BibView*msgText55.label:  Datei konnte nicht geoeffnet werden!",
"BibView*msgText56.label:  Dateiname nicht korrekt!",
"BibView*msgText60.label:  Ueberpruefung ergab Inkonsistenzen.\\nFehlerhafte Karten werden in Liste angezeigt.",
"BibView*msgText61.label:  Ueberpruefung ergab keine Inkonsistenzen.",
"BibView*msgText62.label:  Ueberpruefung ergab Inkonsistenzen.\\nFehler in markierter Zeile korrigieren und erneut laden.",
"BibView*msgText63.label:  Einige zwingende Felder wurden nicht ausgefuellt.\\nDer Eintrag wurde trotzdem gespeichert.",
"BibView*msgText64.label:  Syntax Fehler: Einige Eintraege koennten verloren gehen.",
"BibView*msgText100.label: Doppelte Schluessel nicht erlaubt.\\nKarte evtl. bereits im Baum enthalten.",
"BibView*msgText101.label: Kein regulaerer Ausdruck.",
"BibView*msgText102.label: Keine Eintraege in der BibTeX-Datei gefunden.",
"BibView*msgText114.label: Card incomplete: no author or editor.",
"BibView*msgText115.label: Card incomplete: no chapter and/or pages.",
"BibView*msgText120.label: Keine weiteren Dateien ladbar.\\nErst schliessen.",
"BibView*msgText121.label: Interner Fehler: Nr. 121",
"BibView*quit*accelerators:\
	<Key>q: notify()",
"BibView*mainwin*services*accelerators:\
	<Key>i: notify()",
"BibView*mainwin*options*accelerators:\
	<Key>o: notify()",
"BibView*mainwin*window*accelerators:\
	<Key>f: notify()",
"BibView*mainwin*help*accelerators:\
	<Key>h: notify()",
