"BibView.title:		bibView",
"BibView.iconName:		bibView",
"BibView*font: *helvetica-bold-r-*-120-*",
"BibView*mainwin.orientation:		vertical",
"BibView*mainwin.height:			150",
"BibView*mainwin.width:			240",
"BibView*mainwin*showGrip:		false",
"BibView*mainwin.commandBox.width: 	500",
"BibView*mainwin.commandBox.vSpace: 	0",
"BibView*mainwin.commandBox.hSpace: 	0",
"BibView*mainwin.commandBox.min: 	21",
"BibView*mainwin.desktop.label:",
"BibView*mainwin.desktop.width:   	500",
"BibView*mainwin.desktop.height:    	150",
"BibView*mainwin.desktop.fromVert:  	commandBox",
"BibView*mainwin*file.label:		File",
"BibView*mainwin*file.menuName:		fileMenu",
"BibView*fileMenu.label:			File Menu",
"BibView*fileMenu*item1*label:		New",
"BibView*fileMenu*item2*label:		Open",
"BibView*fileMenu*item3*label:		Close",
"BibView*fileMenu*item4*label:		Save",
"BibView*fileMenu*item5*label:		Save as",
"BibView*fileMenu*quit*label:		Quit",
"BibView*mainwin*services.label:      	Services",
"BibView*mainwin*services.menuName:	servicesMenu",
"BibView*servicesMenu.label:		Services Menu",
"BibView*servicesMenu*item1*label:	Consistency Check",
"BibView*servicesMenu*item2*label:	Unify",
"BibView*servicesMenu*item3*label:	Print",
"BibView*servicesMenu*item4*label:	Edit Macros",
"BibView*servicesMenu*item5*label:	Load Configuration",
"BibView*mainwin*options.label:       	Options",
"BibView*mainwin*options.menuName:	optionsMenu",
"BibView*mainwin*options*leftMargin: 	20",
"BibView*optionsMenu.label:		Options Menu",
"BibView*optionsMenu*item1*label:	Beep on Error",
"BibView*optionsMenu*item2*label:	Backup  ",
"BibView*optionsMenu*item3*label:	Icons on Desktop  ",
"BibView*optionsMenu*item4*label:	Automatic Check",
"BibView*optionsMenu*item5*label:	Required Fields",
"BibView*optionsMenu*item6*label:	Ignore Case",
"BibView*optionsMenu*item7*label:	Print As Bib",
"BibView*optionsMenu*item8*label:	Display Error Window",
"BibView*mainwin*window.label:      	Window",
"BibView*mainwin*window.menuName:	windowMenu",
"BibView*mainwin*window*leftMargin: 	20",
"BibView*windowMenu.label:		Window Menu",
"BibView*windowMenu*item1*label:		Cascade",
"BibView*mainwin*help.label:      	Help",
"BibView*mainwin*help.menuName:		helpMenu",
"BibView*helpMenu.label:			Help Menu",
"BibView*helpMenu*item1*label:		Survey",
"BibView*helpMenu*item2*label:		Main Window",
"BibView*helpMenu*item3*label:		Bibliography Window",
"BibView*helpMenu*item4*label:		List Window",
"BibView*helpMenu*item5*label:		Card Window",
"BibView*helpMenu*item6*label:		Other Windows",
"BibView*helpMenu*info*label:		Program Info",
"BibView*bibWin.orientation:		vertical",
"BibView*bibWin.width:			352",
"BibView*bibWin.height:			150",
"BibView*bibWin*showGrip:		false",
"BibView*bibWin*commandBox.width: 	450",
"BibView*bibWin*commandBox.vSpace: 	0",
"BibView*bibWin*commandBox.hSpace: 	0",
"BibView*bibWin*commandBox.min: 		21",
"BibView*bibWin*commandBox*new.label:	New",
"BibView*bibWin*new.menuName:		newMenu",
"BibView*bibWin*newMenu.label:		New Menu",
"BibView*bibWin*commandBox*search.label:	Search",
"BibView*bibWin*commandBox*list.label:	List",
"BibView*bibWin*commandBox*macro.label:	Macros",
"BibView*bibWin*commandBox*print.label:	Print",
"BibView*bibWin*commandBox*save.label:	Save",
"BibView*bibWin*commandBox*close.label:	Close",
"BibView*bibWin*window.label:		Window",
"BibView*bibWin*window.menuName:		windowMenu",
"BibView*bibWin*window*leftMargin: 	20",
"BibView*bibWin*windowMenu.label:	Window-Menu",
"BibView*bibWin*windowMenu*item1.label:	Cascade",
"BibView*bibWin.bibdesk.label:",
"BibView*bibWin.bibdesk.width:   	450",
"BibView*bibWin.bibdesk.height:    	350",
"BibView*bibWin.bibdesk.fromVert:  	commandBox",
"BibView*listWin*allowShellResize:	true",
"BibView*listWin*allowResize:		true",
"BibView*listWin.height:			650",
"BibView*listWin.width:			530",
"BibView*listWin.horizDistance:		235",
"BibView*listWin.vertDistance:		20",
"BibView*listWin.borderWidth:		2",
"BibView*listWin.orientation:		vertical",
"BibView*listWin*showGrip:		false",
"BibView*listWin.commandBox.vSpace: 	0",
"BibView*listWin.commandBox.hSpace: 	0",
"BibView*listWin.commandBox.min: 	21",
"BibView*listWin.commandBox*print.label:	Print",
"BibView*listWin.commandBox*save.label:	Save",
"BibView*listWin.commandBox*sort.label:	Sort",
"BibView*listWin.commandBox*copy.label:	Copy",
"BibView*listWin.commandBox*quit.label:	Close",
"BibView*listWin*sort.menuName:		sortMenu",
"BibView*listWin*sortMenu.label:		Sort Menu",
"BibView*listWin*listVport.fromVert:	commandBox",
"BibView*listWin*listVport*font: 	*fixed-bold-r-*-100-*",
"BibView*listWin*head1.width:		1200", 
"BibView*listWin*list.verticalList:	true",
"BibView*listWin*list.forceColumns:	True",
"BibView*listWin*list.defaultColumns:	1",
"BibView*macWin*allowResize:		true",
"BibView*macWin.height:			300",
"BibView*macWin.width:			450",
"BibView*macWin.borderWidth:		2",
"BibView*macWin.orientation:		vertical",
"BibView*macWin*showGrip:		false",
"BibView*macWin.commandBox.vSpace: 	0",
"BibView*macWin.commandBox.hSpace: 	0",
"BibView*macWin.commandBox.min: 	21",
"BibView*macWin.commandBox*save.label:	Save",
"BibView*macWin.commandBox*close.label:	Close",
"BibView*macText.fromVert:		commandBox",
"BibView*macText*font: 			*fixed-bold-r-*-100-*",
"BibView*macText*textSource.editType:	edit",
"BibView*macText*input: 			true",
"BibView*helpWin*allowResize:		true",
"BibView*helpWin.height:			300",
"BibView*helpWin.width:			500",
"BibView*helpWin.borderWidth:		2",
"BibView*helpWin.orientation:		vertical",
"BibView*helpWin*showGrip:		false",
"BibView*helpWin.commandBox.vSpace: 	0",
"BibView*helpWin.commandBox.hSpace: 	0",
"BibView*helpWin.commandBox.min: 	21",
"BibView*helpWin.commandBox*close.label:	Close",
"BibView*helpText.fromVert:		commandBox",
"BibView*helpText*font: 			*fixed-bold-r-*-100-*",
"BibView*helpText*input: 			true",
"BibView*annoteWin*allowResize:		true",
"BibView*annoteWin.height:		300",
"BibView*annoteWin.width:		500",
"BibView*annoteWin.borderWidth:		2",
"BibView*annoteWin.orientation:		vertical",
"BibView*annoteWin*showGrip:		false",
"BibView*annoteWin.commandBox.vSpace: 	0",
"BibView*annoteWin.commandBox.hSpace: 	0",
"BibView*annoteWin.commandBox.min: 	21",
"BibView*annoteWin.commandBox*close.label:	Close",
"BibView*annoteWin.commandBox*save.label:	Save",
"BibView*annoteText.fromVert:		commandBox",
"BibView*annoteText*font: 			*fixed-bold-r-*-100-*",
"BibView*annoteText*input: 			true",
"BibView*srchShell.title:		Search",
"BibView*srchWin.height:			400",
"BibView*srchWin.width:			380",
"BibView*srchWin.borderWidth:		2",
"BibView*srchWin.orientation:		vertical",
"BibView*srchWin*showGrip:		false",
"BibView*srchWin.commandBox.vSpace: 	0",
"BibView*srchWin.commandBox.hSpace: 	0",
"BibView*srchWin.commandBox.min: 	21",
"BibView*srchWin.commandBox*ok.label:	Start search",
"BibView*srchWin.commandBox*cancel.label:Cancel",
"BibView*srchWin*srchFlds*borderwidth:	0",
"BibView*srchWin*srchFlds*vSpace:	0",
"BibView*srchWin*srchFlds*font: 		*helvetica-bold-r-*-100-*",
"BibView*srchWin*ComboBox.borderWidth:	0",
"BibView*srchWin*ComboBox.margin:	4",
"BibView*srchWin*ComboBox*Text.width:	250",
"BibView*srchWin*ComboBox*Toggle.width:	100",
"BibView*srchWin*ComboBox*Toggle.justify:left",
"BibView*srchWin*ComboBox*Viewport.borderWidth:	0",
"BibView*srchWin*ComboBox*Viewport.width:	300",
"BibView*srchWin*ComboBox*Viewport.height:	100",
"BibView*srchWin*cbAllFields*name:	AllFields:",
"BibView*srchWin*cbBibType*name:	BibType:",
"BibView*srchWin*cbMainkey*name:		BibTeX-Key:",
"BibView*srchWin*cbUserfield1.width:	100",
"BibView*srchWin*cbUserfield2.width:	100",
"BibView*srchWin*cbUsercont1.width:	250",
"BibView*srchWin*cbUsercont2.width:	250",
"BibView*srchWin*fldHead.label:	        Userdefined Fields",
"BibView*srchWin*fldHead.width:	        355",
"BibView*cardShell.allowShellResize:	true",
"BibView*cardWin*Text*scrollHorizontal:	whenNeeded",
"BibView*cardWin*Text*scrollVertical:	whenNeeded",
"BibView*cardWin*Text*thickness:		8",
"BibView*cardWin*Text.height:		28",
"BibView*cardWin*Text*textSource.editType:	edit",
"BibView*cardWin*Text*input: 		true",
"BibView*cardWin*Label*justify:		left",
"BibView*cardWin.width:			450",
"BibView*cardWin.borderWidth:		2",
"BibView*cardWin.commandBox.showGrip:	false",
"BibView*cardWin.commandBox.width: 	400",
"BibView*cardWin.commandBox.vSpace: 	0",
"BibView*cardWin.commandBox.hSpace: 	0",
"BibView*cardWin.commandBox.min: 	21",
"BibView*cardWin.commandBox.skipAdjust: 	false",
"BibView*cardWin.commandBox*userDef.label:	UserData",
"BibView*cardWin.commandBox*annote.label:	Annote",
"BibView*cardWin.commandBox*save.label:		Save",
"BibView*cardWin.commandBox*change.label:	Change",
"BibView*cardWin.commandBox*delete.label:	Delete",
"BibView*cardWin.commandBox*copy.label:		Copy",
"BibView*cardWin.commandBox*duplicate.label:	Duplicate",
"BibView*cardWin*duplicate.menuName:		duplicateMenu", 
"BibView*cardWin*duplicateMenu.label:		Duplicate Menu",
"BibView*cardWin.commandBox*quit.label:		Close",
"BibView*cardWin.userFldVport*font: 	*helvetica-bold-r-*-100-*",
"BibView*cardWin.userFldVport.allowResize: true",
"BibView*cardWin.userFldVport.min:	100",
"BibView*cardWin.userFldVport.max:	150",
"BibView*cardWin*userFieldL.width:	150",
"BibView*cardWin*userField.width:	225",
"BibView*cardWin*userFldHead.label:	Userdefined Fields",
"BibView*cardWin*userFldHead.width:	380",
"BibView*cardWin*userFldHead.borderWidth:	2",
"BibView*cardWin*userFldHead.justify:	center",
"BibView*cardWin.carddesk*font: 		*helvetica-bold-r-*-100-*",
"BibView*cardWin.carddesk*Label*borderWidth:	0",
"BibView*cardWin.carddesk.defaultDistance:	0",
"BibView*cardWin.carddesk*field.width:		355",
"BibView*cardWin.carddesk*field.height:		30",
"BibView*confirmLoadShell.title:		Load?",
"BibView*confirmLoadBox.label:		File is loaded.\\nTake saved version?",
"BibView*confirmLoadBox*ok.label:	OK",
"BibView*confirmLoadBox*cancel.label:	Cancel",
"BibView*confirmQuitShell.title:	Close?",
"BibView*confirmQuitBox.label:	There are unsaved changes!\\nDo you really want to close?",
"BibView*confirmQuitBox*ok.label:	Close",
"BibView*confirmQuitBox*cancel.label:	Cancel",
"BibView*confirmMacQuitBox*quit.label:	Quit",
"BibView*fileSelectBoxShell.title:	BibView Filemanager",
"BibView*fileSelectBoxShell.iconName:	BibView Filemanager",
"BibView*fileLoadHead.label:		Choose BibTeX-file:",
"BibView*fileRenameHead.label:		Rename BibTeX-file:",
"BibView*filePrintHead.label:		LaTeX file:",
"BibView*configLoadHead.label:		Configuration file:",
"BibView*fileListSaveHead.label:		Save list as BibTeX-file:",
"BibView*bibselShell.title:		BibView-file-Selection",
"BibView*bibselWin*closeHead.label:	Close",
"BibView*bibselWin*saveHead.label:	Save",
"BibView*bibselWin*saveAsHead.label:	Save as (file that should be saved",
"BibView*bibselWin*copyHead.label:	Copy to",
"BibView*bibselWin*collSrcHead.label:	Take entries from",
"BibView*bibselWin*collDestHead.label:	Insert into",
"BibView*bibselWin*checkHead.label:	Check",
"BibView*bibselWin*printHead.label:	Print",
"BibView*bibselWin*macroHead.label:	Edit Macros",
"BibView*bibselWin*cancel.label:		Cancel",
"BibView*errorShell.title:	BibView Message",
"BibView*errorWin*type.label:	ERROR:",
"BibView*errorWin*ok.label:	OK",
"BibView*warnShell.title:	BibView Message",
"BibView*warnWin*type.label:	WARNING:",
"BibView*warnWin*ok.label:	OK",
"BibView*noticeShell.title:	BibView Message",
"BibView*noticeWin*type.label:	HINT:",
"BibView*noticeWin*ok.label:	OK",
"BibView*msgText10.label:  Not yet implemented.",
"BibView*msgText11.label:  No storage!",
"BibView*msgText12.label:  No BibTeX file!",
"BibView*msgText14.label:  No BibTeX file opened.",
"BibView*msgText15.label:  File cannot be unified!.",
"BibView*msgText16.label:  Card cannot be copied.",
"BibView*msgText17.label:  List cannot be copied.",
"BibView*msgText30.label:  Windows can't be cascaded!",
"BibView*msgText50.label:  File can't be opened!",
"BibView*msgText51.label:  No Configuration file.",
"BibView*msgText52.label:  Temporary file can't be opened!",
"BibView*msgText53.label:  Macros can't be saved!",
"BibView*msgText54.label:  Please close Error-Window!",
"BibView*msgText55.label:  File can't be saved!",
"BibView*msgText60.label:  There are inconsistencies.\\nThe cards are shown in List.",
"BibView*msgText61.label:  No inconsistencies.",
"BibView*msgText62.label:  There are syntactic problems.\\n Please correct error and load again.",
"BibView*msgText63.label:  Some required fields are not entered.\\nThe entry was saved.",
"BibView*msgText64.label:  Syntax Error! Entries will be lost.",
"BibView*msgText100.label: Double Keys are not allowed.\\nCard in Tree.",
"BibView*msgText101.label: No regular expression.",
"BibView*msgText102.label: No entries found in file.",
"BibView*msgText114.label: Card incomplete: no author or editor.",
"BibView*msgText115.label: Card incomplete: no chapter and/or page.",
"BibView*msgText120.label: No further Files loadable.\\nPlease close bibliography.",
"BibView*msgText121.label: Internal Bug: Nr. 121",
/*"BibView*quit.accelerators:\ */
"BibView*fileMenu*quit.accelerators:\
	<Key>q: notify()", 
"BibView*listWin.commandBox*quit.accelerators:\
	<Key>q: notify()", 
"BibView*mainwin*services*accelerators:\
	<Key>i: notify()",
"BibView*mainwin*options*accelerators:\
	<Key>o: notify()",
"BibView*mainwin*window*accelerators:\
	<Key>f: notify()",
"BibView*mainwin*help*accelerators:\
	<Key>h: notify()",
"BibView*mainwin*help*accelerators:\
	<Key>h: notify()",
